@@ -1,5 +1,6 @@
 use crate::ber::*;
 use crate::*;
+use alloc::borrow::Cow;
 use alloc::string::String;
 use core::convert::TryInto;
 
@@ -9,7 +10,7 @@ use core::convert::TryInto;
 /// It contains a header, and either a reference to or owned data for the object content.
 ///
 /// Note: this type is only provided in **borrowed** version (*i.e.* it can own the inner data).
-#[derive(Debug)]
+#[derive(Clone, Debug, PartialEq)]
 pub struct Any<'a> {
     /// The object header
     pub header: Header<'a>,
@@ -32,7 +33,7 @@ impl<'a> Any<'a> {
             header: Header {
                 tag,
                 constructed,
-                class: crate::Class::Universal,
+                class: Class::Universal,
                 length: Length::Definite(data.len()),
                 raw_tag: None,
             },
@@ -46,19 +47,22 @@ impl<'a> Any<'a> {
         self.header.class
     }
 
+    /// Update the class of the current object
+    #[inline]
+    pub fn with_class(self, class: Class) -> Self {
+        Any {
+            header: self.header.with_class(class),
+            ..self
+        }
+    }
+
     /// Return the `Tag` of this object
     #[inline]
     pub const fn tag(&self) -> Tag {
         self.header.tag
     }
 
-    /// Get the bytes representation of the *content*
-    #[inline]
-    pub fn as_bytes(&'a self) -> &'a [u8] {
-        &self.data
-    }
-
-    /// Convert the current object to an object with same contents but different tag
+    /// Update the tag of the current object
     #[inline]
     pub fn with_tag(self, tag: Tag) -> Self {
         Any {
@@ -67,12 +71,18 @@ impl<'a> Any<'a> {
         }
     }
 
+    /// Get the bytes representation of the *content*
+    #[inline]
+    pub fn as_bytes(&'a self) -> &'a [u8] {
+        self.data
+    }
+
     #[inline]
     pub fn parse_ber<T>(&'a self) -> ParseResult<'a, T>
     where
         T: FromBer<'a>,
     {
-        T::from_ber(&self.data)
+        T::from_ber(self.data)
     }
 
     #[inline]
@@ -80,7 +90,7 @@ impl<'a> Any<'a> {
     where
         T: FromDer<'a>,
     {
-        T::from_der(&self.data)
+        T::from_der(self.data)
     }
 }
 
@@ -122,7 +132,8 @@ impl<'a> Any<'a> {
     /// Attempt to convert object to `Oid` (ASN.1 type: `RELATIVE-OID`).
     pub fn relative_oid(self) -> Result<Oid<'a>> {
         self.header.assert_tag(Tag::RelativeOid)?;
-        self.try_into()
+        let asn1 = Cow::Borrowed(self.data);
+        Ok(Oid::new_relative(asn1))
     }
     impl_any_into!(printablestring => PrintableString<'a>, "PrintableString");
     impl_any_into!(sequence => Sequence<'a>, "SEQUENCE");
@@ -161,6 +172,7 @@ impl<'a> FromDer<'a> for Any<'a> {
 impl CheckDerConstraints for Any<'_> {
     fn check_constraints(any: &Any) -> Result<()> {
         any.header.length().assert_definite()?;
+        // if len < 128, must use short form (10.1: minimum number of octets)
         Ok(())
     }
 }
@@ -202,13 +214,13 @@ impl ToDer for Any<'_> {
     }
 
     fn write_der_content(&self, writer: &mut dyn std::io::Write) -> SerializeResult<usize> {
-        writer.write(&self.data).map_err(Into::into)
+        writer.write(self.data).map_err(Into::into)
     }
 
     /// Similar to using `to_der`, but uses header without computing length value
     fn write_der_raw(&self, writer: &mut dyn std::io::Write) -> SerializeResult<usize> {
         let sz = self.header.write_der_header(writer)?;
-        let sz = sz + writer.write(&self.data)?;
+        let sz = sz + writer.write(self.data)?;
         Ok(sz)
     }
 }
