@@ -1,10 +1,11 @@
 use crate::datetime::decode_decimal;
 use crate::*;
 use alloc::format;
-#[cfg(feature = "datetime")]
-use chrono::{DateTime, TimeZone, Utc};
+use alloc::string::String;
 use core::convert::TryFrom;
 use core::fmt;
+#[cfg(feature = "datetime")]
+use time::OffsetDateTime;
 
 #[derive(Debug, PartialEq, Eq, PartialOrd, Ord)]
 pub struct GeneralizedTime(pub ASN1DateTime);
@@ -77,6 +78,7 @@ impl GeneralizedTime {
             [b'.' | b',', rem @ ..] => {
                 let mut fsecond = 0;
                 let mut rem = rem;
+                let mut digits = 0;
                 for idx in 0..=4 {
                     if rem.is_empty() {
                         if idx == 0 {
@@ -85,6 +87,7 @@ impl GeneralizedTime {
                                 "malformed time string (dot or comma but no digits)",
                             ));
                         }
+                        digits = idx;
                         break;
                     }
                     if idx == 4 {
@@ -94,10 +97,11 @@ impl GeneralizedTime {
                     }
                     match rem[0] {
                         b'0'..=b'9' => {
-                            // XXX check for overflow in mul
-                            fsecond = fsecond * 10 + (rem[0] - b'0') as u32;
+                            // cannot overflow, max 4 digits will be read
+                            fsecond = fsecond * 10 + (rem[0] - b'0') as u16;
                         }
                         b'Z' | b'+' | b'-' => {
+                            digits = idx;
                             break;
                         }
                         _ => {
@@ -108,6 +112,13 @@ impl GeneralizedTime {
                     }
                     rem = &rem[1..];
                 }
+                // fix fractional seconds depending on the number of digits
+                // for ex, date "xxxx.3" means 3000 milliseconds, not 3
+                let fsecond = match digits {
+                    1 => fsecond * 100,
+                    2 => fsecond * 10,
+                    _ => fsecond,
+                };
                 (Some(fsecond), rem)
             }
             _ => (None, rem),
@@ -131,12 +142,12 @@ impl GeneralizedTime {
             [b'+', h1, h2, m1, m2] => {
                 let hh = decode_decimal(Self::TAG, *h1, *h2)?;
                 let mm = decode_decimal(Self::TAG, *m1, *m2)?;
-                ASN1TimeZone::Offset(1, hh, mm)
+                ASN1TimeZone::Offset(hh as i8, mm as i8)
             }
             [b'-', h1, h2, m1, m2] => {
                 let hh = decode_decimal(Self::TAG, *h1, *h2)?;
                 let mm = decode_decimal(Self::TAG, *m1, *m2)?;
-                ASN1TimeZone::Offset(-1, hh, mm)
+                ASN1TimeZone::Offset(-(hh as i8), mm as i8)
             }
             _ => return Err(Self::TAG.invalid_value("malformed time string: no time zone")),
         };
@@ -155,11 +166,8 @@ impl GeneralizedTime {
     /// Return a ISO 8601 combined date and time with time zone.
     #[cfg(feature = "datetime")]
     #[cfg_attr(docsrs, doc(cfg(feature = "datetime")))]
-    pub fn utc_datetime(&self) -> DateTime<Utc> {
-        let dt = &self.0;
-        // XXX Utc only if Z
-        Utc.ymd(dt.year as i32, dt.month as u32, dt.day as u32)
-            .and_hms(dt.hour as u32, dt.minute as u32, dt.second as u32)
+    pub fn utc_datetime(&self) -> Result<OffsetDateTime> {
+        self.0.to_datetime()
     }
 }
 
@@ -176,7 +184,7 @@ impl<'a> TryFrom<Any<'a>> for GeneralizedTime {
             return Err(Error::StringInvalidCharset);
         }
 
-        GeneralizedTime::from_bytes(&any.data)
+        GeneralizedTime::from_bytes(any.data)
     }
 }
 
@@ -185,7 +193,7 @@ impl fmt::Display for GeneralizedTime {
         let dt = &self.0;
         let fsec = match self.0.millisecond {
             Some(v) => format!(".{}", v),
-            None => format!(""),
+            None => String::new(),
         };
         match dt.tz {
             ASN1TimeZone::Undefined => write!(
@@ -195,14 +203,14 @@ impl fmt::Display for GeneralizedTime {
             ),
             ASN1TimeZone::Z => write!(
                 f,
-                "{:04}-{:02}-{:02} {:02}:{:02}:{:02}{} Z",
+                "{:04}-{:02}-{:02} {:02}:{:02}:{:02}{}Z",
                 dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second, fsec
             ),
-            ASN1TimeZone::Offset(sign, hh, mm) => {
-                let s = if sign > 0 { '+' } else { '-' };
+            ASN1TimeZone::Offset(hh, mm) => {
+                let (s, hh) = if hh > 0 { ('+', hh) } else { ('-', -hh) };
                 write!(
                     f,
-                    "{:04}-{:02}-{:02} {:02}:{:02}:{:02}{} {}{:02}{:02}",
+                    "{:04}-{:02}-{:02} {:02}:{:02}:{:02}{}{}{:02}{:02}",
                     dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second, fsec, s, hh, mm
                 )
             }
@@ -214,13 +222,13 @@ impl<'a> CheckDerConstraints for GeneralizedTime {
     fn check_constraints(any: &Any) -> Result<()> {
         // X.690 section 11.7.1: The encoding shall terminate with a "Z"
         if any.data.last() != Some(&b'Z') {
-            return Err(Error::DerConstraintFailed);
+            return Err(Error::DerConstraintFailed(DerConstraint::MissingTimeZone));
         }
         // X.690 section 11.7.2: The seconds element shall always be present.
         // XXX
         // X.690 section 11.7.4: The decimal point element, if present, shall be the point option "."
         if any.data.iter().any(|&b| b == b',') {
-            return Err(Error::DerConstraintFailed);
+            return Err(Error::DerConstraintFailed(DerConstraint::MissingSeconds));
         }
         Ok(())
     }
