@@ -180,7 +180,7 @@ impl<'a> Set<'a> {
     {
         match self.content {
             Cow::Borrowed(b) => f(b),
-            _ => Err(nom::Err::Failure(Error::LifetimeError)),
+            _ => Err(nom::Err::Error(Error::LifetimeError)),
         }
     }
 
