@@ -13,7 +13,8 @@ where
         any.tag().assert_eq(Tag(TAG))?;
         any.header.assert_constructed()?;
         if any.class() as u8 != CLASS {
-            return Err(Error::UnexpectedClass(any.class()));
+            let class = Class::try_from(CLASS).ok();
+            return Err(Error::unexpected_class(class, any.class()));
         }
         let (_, inner) = T::from_ber(any.data)?;
         Ok(TaggedValue::explicit(inner))
@@ -27,7 +28,7 @@ where
 {
     fn check_constraints(any: &Any) -> Result<()> {
         any.header.length.assert_definite()?;
-        let (_, inner) = Any::from_ber(&any.data)?;
+        let (_, inner) = Any::from_ber(any.data)?;
         T::check_constraints(&inner)?;
         Ok(())
     }
@@ -186,7 +187,7 @@ where
 {
     fn check_constraints(any: &Any) -> Result<()> {
         any.header.length.assert_definite()?;
-        let (_, inner_any) = Any::from_der(&any.data)?;
+        let (_, inner_any) = Any::from_der(any.data)?;
         T::check_constraints(&inner_any)?;
         Ok(())
     }
