@@ -3,13 +3,41 @@ use alloc::str;
 use alloc::string;
 use alloc::string::String;
 use displaydoc::Display;
-use nom::error::{ErrorKind, ParseError};
+use nom::error::{ErrorKind, FromExternalError, ParseError};
 use nom::IResult;
 #[cfg(feature = "std")]
 use std::io;
 #[cfg(feature = "std")]
 use thiserror::Error;
 
+#[cfg(feature = "std")]
+impl std::error::Error for DerConstraint {}
+
+#[derive(Clone, Copy, Debug, Display, PartialEq)]
+/// Error types for DER constraints
+pub enum DerConstraint {
+    /// Indefinite length not allowed
+    IndefiniteLength,
+    /// Object must not be constructed
+    Constructed,
+    /// Object must be constructed,
+    NotConstructed,
+    /// DateTime object is missing timezone
+    MissingTimeZone,
+    /// DateTime object is missing seconds
+    MissingSeconds,
+    /// Bitstring unused bits must be set to zero
+    UnusedBitsNotZero,
+    /// Boolean value must be 0x00 of 0xff
+    InvalidBoolean,
+    /// Integer must not be empty
+    IntegerEmpty,
+    /// Leading zeroes in Integer encoding
+    IntegerLeadingZeroes,
+    /// Leading 0xff in negative Integer encoding
+    IntegerLeadingFF,
+}
+
 // XXX
 // thiserror does not work in no_std
 // see https://github.com/dtolnay/thiserror/pull/64
@@ -19,9 +47,13 @@ impl std::error::Error for Error {}
 
 /// The error type for operations of the [`FromBer`](crate::FromBer),
 /// [`FromDer`](crate::FromDer), and associated traits.
-#[derive(Debug, Display, PartialEq)]
+#[derive(Clone, Debug, Display, PartialEq)]
 // #[cfg_attr(feature = "std", derive(Error))]
 pub enum Error {
+    /// BER object does not have the expected type
+    BerTypeError,
+    /// BER object does not have the expected value
+    BerValueError,
     /// Invalid Length
     InvalidLength,
     /// Invalid Value when parsing object with tag {tag:?} {msg:}
@@ -32,8 +64,11 @@ pub enum Error {
     UnknownTag(u32),
     /// Unexpected Tag (expected: {expected:?}, actual: {actual:?})
     UnexpectedTag { expected: Option<Tag>, actual: Tag },
-    /// Unexpected Class (expected: {0:?}
-    UnexpectedClass(Class),
+    /// Unexpected Class (expected: {expected:?}, actual: {actual:?})
+    UnexpectedClass {
+        expected: Option<Class>,
+        actual: Class,
+    },
 
     /// Indefinite length not allowed
     IndefiniteLengthUnexpected,
@@ -52,9 +87,11 @@ pub enum Error {
 
     /// Invalid encoding or forbidden characters in string
     StringInvalidCharset,
+    /// Invalid Date or Time
+    InvalidDateTime,
 
     /// DER Failed constraint
-    DerConstraintFailed,
+    DerConstraintFailed(DerConstraint),
 
     /// Requesting borrowed data from a temporary object
     LifetimeError,
@@ -68,6 +105,26 @@ pub enum Error {
     NomError(ErrorKind),
 }
 
+impl Error {
+    /// Build an error from the provided invalid value
+    #[inline]
+    pub const fn invalid_value(tag: Tag, msg: String) -> Self {
+        Self::InvalidValue { tag, msg }
+    }
+
+    /// Build an error from the provided unexpected class
+    #[inline]
+    pub const fn unexpected_class(expected: Option<Class>, actual: Class) -> Self {
+        Self::UnexpectedClass { expected, actual }
+    }
+
+    /// Build an error from the provided unexpected tag
+    #[inline]
+    pub const fn unexpected_tag(expected: Option<Tag>, actual: Tag) -> Self {
+        Self::UnexpectedTag { expected, actual }
+    }
+}
+
 impl<'a> ParseError<&'a [u8]> for Error {
     fn from_error_kind(_input: &'a [u8], kind: ErrorKind) -> Self {
         Error::NomError(kind)
@@ -79,7 +136,7 @@ impl<'a> ParseError<&'a [u8]> for Error {
 
 impl From<Error> for nom::Err<Error> {
     fn from(e: Error) -> Self {
-        nom::Err::Failure(e)
+        nom::Err::Error(e)
     }
 }
 
@@ -110,6 +167,12 @@ impl From<nom::Err<Error>> for Error {
     }
 }
 
+impl<I, E> FromExternalError<I, E> for Error {
+    fn from_external_error(_input: I, kind: ErrorKind, _e: E) -> Error {
+        Error::NomError(kind)
+    }
+}
+
 /// Holds the result of BER/DER serialization functions
 pub type ParseResult<'a, T> = IResult<&'a [u8], T, Error>;
 
