@@ -69,7 +69,7 @@ impl<'a> Header<'a> {
 
     /// Set the length of this `Header`
     #[inline]
-    pub fn with_lenth(self, length: Length) -> Self {
+    pub fn with_length(self, length: Length) -> Self {
         Self { length, ..self }
     }
 
@@ -124,11 +124,7 @@ impl<'a> Header<'a> {
     /// Return error if class is not the expected class
     #[inline]
     pub const fn assert_class(&self, class: Class) -> Result<()> {
-        if self.class as u8 == class as u8 {
-            Ok(())
-        } else {
-            Err(Error::UnexpectedClass(class))
-        }
+        self.class.assert_eq(class)
     }
 
     /// Return error if tag is not the expected tag
@@ -156,6 +152,27 @@ impl<'a> Header<'a> {
             Err(Error::ConstructExpected)
         }
     }
+
+    /// Test if object class is Universal
+    #[inline]
+    pub const fn is_universal(&self) -> bool {
+        self.class as u8 == Class::Universal as u8
+    }
+    /// Test if object class is Application
+    #[inline]
+    pub const fn is_application(&self) -> bool {
+        self.class as u8 == Class::Application as u8
+    }
+    /// Test if object class is Context-specific
+    #[inline]
+    pub const fn is_contextspecific(&self) -> bool {
+        self.class as u8 == Class::ContextSpecific as u8
+    }
+    /// Test if object class is Private
+    #[inline]
+    pub const fn is_private(&self) -> bool {
+        self.class as u8 == Class::Private as u8
+    }
 }
 
 impl From<Tag> for Header<'_> {
@@ -242,7 +259,9 @@ impl<'a> FromDer<'a> for Header<'a> {
             }
             (_, 0) => {
                 // Indefinite form is not allowed in DER (10.1)
-                return Err(::nom::Err::Error(Error::IndefiniteLengthUnexpected));
+                return Err(::nom::Err::Error(Error::DerConstraintFailed(
+                    DerConstraint::IndefiniteLength,
+                )));
             }
             (_, l1) => {
                 // if len is 0xff -> error (8.1.3.5)
@@ -250,7 +269,11 @@ impl<'a> FromDer<'a> for Header<'a> {
                     return Err(::nom::Err::Error(Error::InvalidLength));
                 }
                 // DER(9.1) if len is 0 (indefinite form), obj must be constructed
-                der_constraint_fail_if!(&i[1..], len.1 == 0 && el.1 != 1);
+                der_constraint_fail_if!(
+                    &i[1..],
+                    len.1 == 0 && el.1 != 1,
+                    DerConstraint::NotConstructed
+                );
                 let (i3, llen) = take(l1)(i2)?;
                 match bytes_to_u64(llen) {
                     Ok(l) => {
@@ -356,7 +379,7 @@ impl ToDer for Header<'_> {
     fn write_der_raw(&self, writer: &mut dyn std::io::Write) -> SerializeResult<usize> {
         // use raw_tag if present
         let sz = match &self.raw_tag {
-            Some(t) => writer.write(&t)?,
+            Some(t) => writer.write(t)?,
             None => (self.class, self.constructed, self.tag).write_der_header(writer)?,
         };
         let sz = sz + self.length.write_der_header(writer)?;
