@@ -42,6 +42,10 @@
 //! combination to objects and methods from this crate. Reading the nom documentation may
 //! help understanding how to write and combine parsers and use the output.
 //!
+//! **Minimum Supported Rust Version**: 1.53.0
+//!
+//! Note: if the `bits` feature is enabled, MSRV is 1.56.0 (due to `bitvec` 1.0)
+//!
 //! ## Examples
 //!
 //! Parse 2 BER integers:
@@ -154,7 +158,7 @@ rust_2018_idioms,*/
 missing_debug_implementations,
 )]
 // pragmas for doc
-#![deny(broken_intra_doc_links)]
+#![deny(rustdoc::broken_intra_doc_links)]
 #![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc(test(
 no_crate_inject,
@@ -172,6 +176,7 @@ mod asn1_types;
 mod ber;
 mod class;
 mod datetime;
+mod derive;
 mod error;
 mod header;
 mod length;
@@ -181,6 +186,7 @@ mod traits;
 pub use asn1_types::*;
 pub use class::*;
 pub use datetime::*;
+pub use derive::*;
 pub use error::*;
 pub use header::*;
 pub use length::*;
@@ -189,3 +195,9 @@ pub use traits::*;
 
 pub use nom;
 pub use nom::{Err, IResult, Needed};
+
+#[doc(hidden)]
+pub mod exports {
+    pub use alloc::borrow;
+    pub use asn1_rs_impl;
+}
