@@ -1,6 +1,8 @@
 use asn1_rs::*;
 use hex_literal::hex;
 use nom::Needed;
+#[cfg(feature = "datetime")]
+use time::macros::datetime;
 
 #[test]
 fn from_ber_any() {
@@ -65,47 +67,59 @@ fn from_ber_generalizedtime() {
     assert_eq!(rem, &[0xff]);
     #[cfg(feature = "datetime")]
     {
-        use chrono::{TimeZone, Utc};
-        let datetime = Utc.ymd(2002, 12, 13).and_hms(14, 29, 23);
+        let datetime = datetime! {2002-12-13 14:29:23 UTC};
 
-        assert_eq!(result.utc_datetime(), datetime);
+        assert_eq!(result.utc_datetime(), Ok(datetime));
     }
+    let _ = result;
     // local time with fractional seconds
     let input = b"\x18\x1019851106210627.3";
     let (rem, result) = GeneralizedTime::from_ber(input).expect("parsing failed");
     assert!(rem.is_empty());
+    assert_eq!(result.0.millisecond, Some(300));
+    assert_eq!(result.0.tz, ASN1TimeZone::Undefined);
     #[cfg(feature = "datetime")]
     {
-        use chrono::{TimeZone, Utc};
-        let datetime = Utc.ymd(1985, 11, 6).and_hms(21, 6, 27);
-        assert_eq!(result.utc_datetime(), datetime);
-        assert_eq!(result.0.millisecond, Some(3));
-        assert_eq!(result.0.tz, ASN1TimeZone::Undefined);
+        let datetime = datetime! {1985-11-06 21:06:27.300_000_000 UTC};
+        assert_eq!(result.utc_datetime(), Ok(datetime));
     }
+    let _ = result;
     // coordinated universal time with fractional seconds
     let input = b"\x18\x1119851106210627.3Z";
     let (rem, result) = GeneralizedTime::from_ber(input).expect("parsing failed");
     assert!(rem.is_empty());
+    assert_eq!(result.0.millisecond, Some(300));
+    assert_eq!(result.0.tz, ASN1TimeZone::Z);
     #[cfg(feature = "datetime")]
     {
-        use chrono::{TimeZone, Utc};
-        let datetime = Utc.ymd(1985, 11, 6).and_hms(21, 6, 27);
-        assert_eq!(result.utc_datetime(), datetime);
-        assert_eq!(result.0.millisecond, Some(3));
-        assert_eq!(result.0.tz, ASN1TimeZone::Z);
+        let datetime = datetime! {1985-11-06 21:06:27.300_000_000 UTC};
+        assert_eq!(result.utc_datetime(), Ok(datetime));
     }
+    let _ = result;
+    // coordinated universal time with fractional seconds
+    let input = b"\x18\x1219851106210627.03Z";
+    let (rem, result) = GeneralizedTime::from_ber(input).expect("parsing failed");
+    assert!(rem.is_empty());
+    assert_eq!(result.0.millisecond, Some(30));
+    assert_eq!(result.0.tz, ASN1TimeZone::Z);
+    #[cfg(feature = "datetime")]
+    {
+        let datetime = datetime! {1985-11-06 21:06:27.03 UTC};
+        assert_eq!(result.utc_datetime(), Ok(datetime));
+    }
+    let _ = result;
     // local time with fractional seconds, and with local time 5 hours retarded in relation to coordinated universal time.
     let input = b"\x18\x1519851106210627.3-0500";
     let (rem, result) = GeneralizedTime::from_ber(input).expect("parsing failed");
     assert!(rem.is_empty());
+    assert_eq!(result.0.millisecond, Some(300));
+    assert_eq!(result.0.tz, ASN1TimeZone::Offset(-5, 0));
     #[cfg(feature = "datetime")]
     {
-        use chrono::{TimeZone, Utc};
-        let datetime = Utc.ymd(1985, 11, 6).and_hms(21, 6, 27);
-        assert_eq!(result.utc_datetime(), datetime);
-        assert_eq!(result.0.millisecond, Some(3));
-        assert_eq!(result.0.tz, ASN1TimeZone::Offset(-1, 5, 0));
+        let datetime = datetime! {1985-11-06 21:06:27.300_000_000 -05:00};
+        assert_eq!(result.utc_datetime(), Ok(datetime));
     }
+    let _ = result;
 }
 
 #[test]
@@ -116,6 +130,14 @@ fn from_ber_int() {
     assert_eq!(rem, &[0xff, 0xff]);
 }
 
+#[test]
+fn from_ber_relative_oid() {
+    let input = &hex!("0d 04 c2 7b 03 02");
+    let (rem, result) = Oid::from_ber_relative(input).expect("parsing failed");
+    assert_eq!(result, Oid::from_relative(&[8571, 3, 2]).unwrap());
+    assert_eq!(rem, &[]);
+}
+
 #[test]
 fn from_ber_length_incomplete() {
     let input = &hex!("30");
