@@ -43,21 +43,26 @@ fn from_der_bitstring() {
     //
     let input = &hex!("03 04 06 6e 5d e0");
     let res = BitString::from_der(input);
-    assert_eq!(res, Err(Err::Failure(Error::DerConstraintFailed)));
+    assert_eq!(
+        res,
+        Err(Err::Error(Error::DerConstraintFailed(
+            DerConstraint::UnusedBitsNotZero
+        )))
+    );
     //
     // long form of length (invalid, < 127)
     //
     // let input = &hex!("03 81 04 06 6e 5d c0");
     // let res = BitString::from_der(input);
-    // assert_eq!(res, Err(Err::Failure(Error::DerConstraintFailed)));
+    // assert_eq!(res, Err(Err::Error(Error::DerConstraintFailed)));
 }
 
 #[test]
 fn from_der_bitstring_constructed() {
     let bytes: &[u8] = &hex!("23 81 0c 03 03 00 0a 3b 03 05 04 5f 29 1c d0");
     assert_eq!(
-        BitString::from_der(&bytes),
-        Err(Err::Failure(Error::ConstructUnexpected))
+        BitString::from_der(bytes),
+        Err(Err::Error(Error::ConstructUnexpected))
     );
 }
 
@@ -84,7 +89,12 @@ fn from_der_bool() {
     //
     let input = &hex!("01 01 7f");
     let res = Boolean::from_der(input);
-    assert_eq!(res, Err(Err::Failure(Error::DerConstraintFailed)));
+    assert_eq!(
+        res,
+        Err(Err::Error(Error::DerConstraintFailed(
+            DerConstraint::InvalidBoolean
+        )))
+    );
 }
 
 #[test]
@@ -102,40 +112,49 @@ fn from_der_generalizedtime() {
     assert_eq!(rem, &[0xff]);
     #[cfg(feature = "datetime")]
     {
-        use chrono::{TimeZone, Utc};
-        let datetime = Utc.ymd(2002, 12, 13).and_hms(14, 29, 23);
-
-        assert_eq!(result.utc_datetime(), datetime);
+        use time::macros::datetime;
+        let datetime = datetime! {2002-12-13 14:29:23 UTC};
+        assert_eq!(result.utc_datetime(), Ok(datetime));
     }
+    let _ = result;
     // local time with fractional seconds (should fail: no 'Z' at end)
     let input = b"\x18\x1019851106210627.3";
     let result = GeneralizedTime::from_der(input).expect_err("should not parse");
-    assert_eq!(result, nom::Err::Failure(Error::DerConstraintFailed));
+    assert_eq!(
+        result,
+        nom::Err::Error(Error::DerConstraintFailed(DerConstraint::MissingTimeZone))
+    );
     // coordinated universal time with fractional seconds
     let input = b"\x18\x1119851106210627.3Z";
     let (rem, result) = GeneralizedTime::from_der(input).expect("parsing failed");
     assert!(rem.is_empty());
+    assert_eq!(result.0.millisecond, Some(300));
+    assert_eq!(result.0.tz, ASN1TimeZone::Z);
     #[cfg(feature = "datetime")]
     {
-        use chrono::{TimeZone, Utc};
-        let datetime = Utc.ymd(1985, 11, 6).and_hms(21, 6, 27);
-        assert_eq!(result.utc_datetime(), datetime);
-        assert_eq!(result.0.millisecond, Some(3));
-        assert_eq!(result.0.tz, ASN1TimeZone::Z);
+        use time::macros::datetime;
+        let datetime = datetime! {1985-11-06 21:06:27.3 UTC};
+        assert_eq!(result.utc_datetime(), Ok(datetime));
     }
+    let _ = result;
     // local time with fractional seconds, and with local time 5 hours retarded in relation to coordinated universal time.
     // (should fail: no 'Z' at end)
     let input = b"\x18\x1519851106210627.3-0500";
     let result = GeneralizedTime::from_der(input).expect_err("should not parse");
-    assert_eq!(result, nom::Err::Failure(Error::DerConstraintFailed));
+    assert_eq!(
+        result,
+        nom::Err::Error(Error::DerConstraintFailed(DerConstraint::MissingTimeZone))
+    );
 }
 
 #[test]
 fn from_der_indefinite_length() {
     let bytes: &[u8] = &hex!("23 80 03 03 00 0a 3b 03 05 04 5f 29 1c d0 00 00");
     assert_eq!(
-        BitString::from_der(&bytes),
-        Err(Err::Error(Error::IndefiniteLengthUnexpected))
+        BitString::from_der(bytes),
+        Err(Err::Error(Error::DerConstraintFailed(
+            DerConstraint::IndefiniteLength
+        )))
     );
 }
 
@@ -187,7 +206,7 @@ fn from_der_oid() {
 fn from_der_optional() {
     let input = &hex!("30 0a 0a 03 00 00 01 02 03 01 00 01");
     let (rem, result) = Sequence::from_der_and_then(input, |input| {
-        let (i, obj0) = <Option<Enumerated>>::from_der(&input)?;
+        let (i, obj0) = <Option<Enumerated>>::from_der(input)?;
         let (i, obj1) = u32::from_der(i)?;
         Ok((i, (obj0, obj1)))
     })
@@ -283,11 +302,12 @@ fn from_der_utctime() {
     assert_eq!(rem, &[0xff]);
     #[cfg(feature = "datetime")]
     {
-        use chrono::{TimeZone, Utc};
-        let datetime = Utc.ymd(2, 12, 13).and_hms(14, 29, 23);
+        use time::macros::datetime;
+        let datetime = datetime! {2-12-13 14:29:23 UTC};
 
-        assert_eq!(result.utc_datetime(), datetime);
+        assert_eq!(result.utc_datetime(), Ok(datetime));
     }
+    let _ = result;
 }
 
 #[test]
